; Domain : Cooking Time (Temporally Expressive)
; Authors : Frederic Maris and Pierre Regnier

(define (domain domain-cooking)
  (:requirements :strips :typing :durative-actions)
  (:types cooking-plate horn - cooking-machine
          noodles lardon - meal
          water-pan oil eggs)
  (:predicates (i)
               (ready ?p - cooking-plate)
               (hot-water ?w - water-pan)
               (hot-oil ?o - oil)
               (cooked ?m - meal)
               (hot ?m - meal)
               (ready-eggs ?e - eggs)
               (mixed3 ?n - noodles ?l - lardon ?e - eggs)
               (eaten-carbonara ?n - noodles ?l - lardon ?e - eggs)

               (fire-cooking-plate-ing ?p - cooking-plate)
               (fire-cooking-plate-enabled-part1 ?p - cooking-plate)
               (fire-cooking-plate-enabled-part2 ?p - cooking-plate)
               (fire-cooking-plate-finished ?p - cooking-plate)

               (boiling-water-ing ?p - cooking-plate  ?w - water-pan)
               (boiling-water-enabled-part1 ?p - cooking-plate  ?w - water-pan)
               (boiling-water-enabled-part2 ?p - cooking-plate  ?w - water-pan)
               (boiling-water-finished ?p - cooking-plate  ?w - water-pan)

               (boiling-oil-ing ?p - cooking-plate  ?o - oil)
               (boiling-oil-enabled-part1 ?p - cooking-plate  ?o - oil)
               (boiling-oil-enabled-part2 ?p - cooking-plate  ?o - oil)
               (boiling-oil-finished ?p - cooking-plate  ?o - oil)

               (cooking-noodles-ing ?w - water-pan  ?n - noodles)
               (cooking-noodles-enabled-part1 ?w - water-pan  ?n - noodles)
               (cooking-noodles-enabled-part2 ?w - water-pan  ?n - noodles)
               (cooking-noodles-finished ?w - water-pan  ?n - noodles)

               (cooking-lardon-ing ?o - oil  ?l - lardon)
               (cooking-lardon-enabled-part1 ?o - oil  ?l - lardon)
               (cooking-lardon-enabled-part2 ?o - oil  ?l - lardon)
               (cooking-lardon-finished ?o - oil  ?l - lardon)
               )

;; #########################################
;; ###          Cooking Actions          ###
;; #########################################

;; ## META-ACTION fire-cooking-plate ##

  (:durative-action fire-cooking-plate
      :parameters (?p - cooking-plate)
      :duration (= ?duration 15.1)
      :condition (at end (fire-cooking-plate-finished ?p))
      :effect (and (at start (fire-cooking-plate-ing ?p))
                   (at start (fire-cooking-plate-enabled-part1 ?p))
                   (at end (not (fire-cooking-plate-ing ?p))))
;;                   (over [(+ start 1) end] (ready ?p))
;;                   (at end (not (ready ?p))))
  )
  (:durative-action fire-cooking-plate-part1
      :parameters (?p - cooking-plate)
      :duration (= ?duration 1)
      :condition (and (over all (fire-cooking-plate-ing ?p))
                      (over all (fire-cooking-plate-enabled-part1 ?p)))
      :effect (and (at end (not (fire-cooking-plate-enabled-part1 ?p)))
                   (at end (fire-cooking-plate-enabled-part2 ?p)))
  )
  (:durative-action fire-cooking-plate-part2
      :parameters (?p - cooking-plate)
      :duration (= ?duration 14)
      :condition (and (over all (fire-cooking-plate-ing ?p))
                      (over all (fire-cooking-plate-enabled-part2 ?p)))
      :effect (and (at start (ready ?p))
                   (at end (not (ready ?p)))
                   (at end (not (fire-cooking-plate-enabled-part2 ?p)))
                   (at end (fire-cooking-plate-finished ?p)))
  )

;; ## META-ACTION boiling-water ##

  (:durative-action boiling-water
      :parameters (?p - cooking-plate  ?w - water-pan)
      :duration (= ?duration 10.1)
      :condition (and (over all (ready ?p))
;;(over [start end[ (ready ?p)))
                      (at end (boiling-water-finished ?p ?w)))
      :effect (and (at start (boiling-water-ing ?p ?w))
                   (at start (boiling-water-enabled-part1 ?p ?w))
                   (at start (not (hot-water ?w))))
;;                   (over [start (+ start 2)[ (not (hot-water ?w)))
;;                   (over [(+ start 2) end] (hot-water ?w)))
  )
  (:durative-action boiling-water-part1
      :parameters (?p - cooking-plate ?w - water-pan)
      :duration (= ?duration 2)
      :condition (and (over all (boiling-water-ing ?p ?w))
                      (over all (boiling-water-enabled-part1 ?p ?w)))
      :effect (and (at end (not (boiling-water-enabled-part1 ?p ?w)))
                   (at end (boiling-water-enabled-part2 ?p ?w)))
  )
  (:durative-action boiling-water-part2
      :parameters (?p - cooking-plate ?w - water-pan)
      :duration (= ?duration 8)
      :condition (and (over all (boiling-water-ing ?p ?w))
                      (over all (boiling-water-enabled-part2 ?p ?w)))
      :effect (and (at start (hot-water ?w))
                   (at end (not (boiling-water-enabled-part2 ?p ?w)))
                   (at end (boiling-water-finished ?p ?w)))
  )

;; ## META-ACTION boiling-oil ##

  (:durative-action boiling-oil
      :parameters (?p - cooking-plate  ?o - oil)
      :duration (= ?duration 10.1)
      :condition (and (over all (ready ?p))
;;(over [start end[ (ready ?p)))
                      (at end (boiling-oil-finished ?p ?o)))
      :effect (and (at start (boiling-oil-ing ?p ?o))
                   (at start (boiling-oil-enabled-part1 ?p ?o))
                   (at start (not (hot-oil ?o))))
;;                   (over [start (+ start 2)[ (not (hot-oil ?o)))
;;                   (over [(+ start 2) end] (hot-oil ?o)))
  )
  (:durative-action boiling-oil-part1
      :parameters (?p - cooking-plate ?o - oil)
      :duration (= ?duration 2)
      :condition (and (over all (boiling-oil-ing ?p ?o))
                      (over all (boiling-oil-enabled-part1 ?p ?o)))
      :effect (and (at end (not (boiling-oil-enabled-part1 ?p ?o)))
                   (at end (boiling-oil-enabled-part2 ?p ?o)))
  )
  (:durative-action boiling-oil-part2
      :parameters (?p - cooking-plate ?o - oil)
      :duration (= ?duration 8)
      :condition (and (over all (boiling-oil-ing ?p ?o))
                      (over all (boiling-oil-enabled-part2 ?p ?o)))
      :effect (and (at start (hot-oil ?o))
                   (at end (not (boiling-oil-enabled-part2 ?p ?o)))
                   (at end (boiling-oil-finished ?p ?o)))
  )

;; ## META-ACTION cooking-noodles ##


  (:durative-action cooking-noodles
      :parameters (?w - water-pan  ?n - noodles)
      :duration (= ?duration 14.1)
      :condition (at end (cooking-noodles-finished ?w ?n))
      :effect (and (at start (cooking-noodles-ing ?w ?n))
                   (at start (cooking-noodles-enabled-part1 ?w ?n))
                   (at start (not (cooked ?n))))
;;                   (over [start (- end 8)[ (not (cooked ?n)))
  )
  (:durative-action cooking-noodles-part1
      :parameters (?w - water-pan ?n - noodles)
      :duration (= ?duration 6)
      :condition (and (over all (hot-water ?w))
;;                      (over [start (- end 8)[ (hot-water ?w)))
                      (over all (cooking-noodles-ing ?w ?n))
                      (over all (cooking-noodles-enabled-part1 ?w ?n)))
      :effect (and (at end (not (cooking-noodles-enabled-part1 ?w ?n)))
                   (at end (cooking-noodles-enabled-part2 ?w ?n)))
  )
  (:durative-action cooking-noodles-part2
      :parameters (?w - water-pan ?n - noodles)
      :duration (= ?duration 8)
      :condition (and (over all (cooking-noodles-ing ?w ?n))
                      (over all (cooking-noodles-enabled-part2 ?w ?n)))
      :effect (and (at start (cooked ?n))
                   (at start (hot ?n))
                   (at end (not (hot ?n)))
;;                   (at (- end 8) (cooked ?n))
;;                   (over [(- end 8) end[ (hot ?n))
;;                   (at end (not (hot ?n))))
                   (at end (not (cooking-noodles-enabled-part2 ?w ?n)))
                   (at end (cooking-noodles-finished ?w ?n)))
  )

;; ## META-ACTION cooking-lardon ##


 (:durative-action cooking-lardon
      :parameters (?o - oil  ?l - lardon)
      :duration (= ?duration 13.1)
      :condition (at end (cooking-lardon-finished ?o ?l))
      :effect (and (at start (cooking-lardon-ing ?o ?l))
                   (at start (cooking-lardon-enabled-part1 ?o ?l))
                   (at start (not (cooked ?l))))
;;                   (over [start (- end 8)[ (not (cooked ?l)))
  )
  (:durative-action cooking-lardon-part1
      :parameters (?o - oil ?l - lardon)
      :duration (= ?duration 5)
      :condition (and (over all (hot-oil ?o))
;;                      (over [start (- end 8)[ (hot-oil ?o)))
                      (over all (cooking-lardon-ing ?o ?l))
                      (over all (cooking-lardon-enabled-part1 ?o ?l)))
      :effect (and (at end (not (cooking-lardon-enabled-part1 ?o ?l)))
                   (at end (cooking-lardon-enabled-part2 ?o ?l)))
  )
  (:durative-action cooking-lardon-part2
      :parameters (?o - oil ?l - lardon)
      :duration (= ?duration 8)
      :condition (and (over all (cooking-lardon-ing ?o ?l))
                      (over all (cooking-lardon-enabled-part2 ?o ?l)))
      :effect (and (at start (cooked ?l))
                   (at start (hot ?l))
                   (at end (not (hot ?l)))
;;                   (at (- end 8) (cooked ?l))
;;                   (over [(- end 8) end[ (hot ?l))
;;                   (at end (not (hot ?l))))
                   (at end (not (cooking-lardon-enabled-part2 ?o ?l)))
                   (at end (cooking-lardon-finished ?o ?l)))
  )

;; ## ACTION breaking-eggs ##

  (:durative-action breaking-eggs
      :parameters (?e - eggs)
      :duration (= ?duration 2)
      :condition (at start (i))
      :effect (and (at end (ready-eggs ?e)))
  )


;; ## ACTION cooking-carbonara ##

  (:durative-action cooking-carbonara
      :parameters (?n - noodles ?l - lardon ?e - eggs)
      :duration (= ?duration 1)
      :condition (and (over all (cooked ?n))
                      (over all (cooked ?l))
                      (over all (ready-eggs ?e)))
;;                      (over [start end[ (cooked ?n))
;;                      (over [start end[ (cooked ?l))
;;                      (over [start end[ (ready ?e)))
      :effect (and (at start (not (mixed3 ?n ?l ?e)))
;; (over [start end[ (not (mixed3 ?n ?l ?e)))
                   (at end (mixed3 ?n ?l ?e)))
  )


;; ########################################
;; ###          Eating Actions          ###
;; ########################################


;; ## ACTION eating-carbonara ##

  (:durative-action eating-carbonara
      :parameters (?n - noodles ?l - lardon ?e - eggs)
      :duration (= ?duration 5)
      :condition (and (over all (mixed3 ?n ?l ?e))
                      (over all (hot ?n))
                      (over all (hot ?l)))
;;                      (over [start end[ (mixed3 ?n ?l ?e))
;;                      (over [start end[ (hot ?n))
;;                      (over [start end[ (hot ?l)))
      :effect (and (at start (not (eaten-carbonara ?n ?l ?e)))
;;                 (over [start end[ (not (eaten-carbonara ?n ?l ?e)))
                   (at end (eaten-carbonara ?n ?l ?e)))
  )

)
